// some parts get from java lexer
grammar TypeNode;

@lexer::header {
package org.napile.asm.io.text.in.type;
}

@header {
package org.napile.asm.io.text.in.type;
}

typeNode: typePart arguments? nulled?;

arguments : '<' typeNode (',' typeNode)? '>';

typePart : classType | typeParameterValue | thisType | methodType | multiType;

// class type
classType : qualifiedName;

// this type
thisType : 'this';

// method type
methodType : '{' '(' methodParameter (',' methodParameter)* ')' ':' returnType '}';

methodParameter : varOrValOrRef Identifier ':' typeNode;

returnType : typeNode;

// multi type
multiType:  '[' multiTypeEntry (',' multiTypeEntry)* ']';

multiTypeEntry : varOrValOrRef Identifier ':' typeNode;

// type parameter value
typeParameterValue : ':' Identifier ':';

varOrValOrRef : 'var' | 'val' | 'ref';

nulled: '?';

qualifiedName: Identifier ('.' Identifier)*;

Identifier : Letter (Letter|JavaIDDigit)*;

fragment Letter :
       '\u0024' |
       '\u0041'..'\u005a' |
       '\u005f' |
       '\u0061'..'\u007a' |
       '\u00c0'..'\u00d6' |
       '\u00d8'..'\u00f6' |
       '\u00f8'..'\u00ff' |
       '\u0100'..'\u1fff' |
       '\u3040'..'\u318f' |
       '\u3300'..'\u337f' |
       '\u3400'..'\u3d2d' |
       '\u4e00'..'\u9fff' |
       '\uf900'..'\ufaff';

fragment JavaIDDigit :
       '\u0030'..'\u0039' |
       '\u0660'..'\u0669' |
       '\u06f0'..'\u06f9' |
       '\u0966'..'\u096f' |
       '\u09e6'..'\u09ef' |
       '\u0a66'..'\u0a6f' |
       '\u0ae6'..'\u0aef' |
       '\u0b66'..'\u0b6f' |
       '\u0be7'..'\u0bef' |
       '\u0c66'..'\u0c6f' |
       '\u0ce6'..'\u0cef' |
       '\u0d66'..'\u0d6f' |
       '\u0e50'..'\u0e59' |
       '\u0ed0'..'\u0ed9' |
       '\u1040'..'\u1049';

WS:  (' '|'\r'|'\t'|'\u000C'|'\n')+ {$channel=HIDDEN;};

COMMENT: '/*' .* '*/' {$channel=HIDDEN;};

LINE_COMMENT: '//' ~('\n'|'\r')* '\r'? '\n' {$channel=HIDDEN;};